# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/19
# Author: ChunTsang <zjun5566@163.com>
"""

import math
from src.dxfgeometry.geometry.line import Line
from src.dxfgeometry.geometry.polygon import Polygon
from src.dxfgeometry.geometry.angle import Angle
from shapely.geometry import Polygon as ShapelyPolygon
from typing import Optional
from src.dxfgeometry.exception.polygon_exception import PolygonAngleException
from src.dxfgeometry.config import default_point_precision


class OrthoPolygon(Polygon):
    _point_list: list[list[float]]
    _outline_dict: dict[float, dict[float, list[Line]]]
    _length_mid_line: Optional[Line]
    _width_mid_line: Optional[Line]
    _angle: Optional[Angle]

    def __init__(self, point_list: list[list[float]], layer_name: str, existing_angle_list: list[float],
                 extra_info=None, point_precision: int = default_point_precision) -> None:
        super(OrthoPolygon, self).__init__(point_list, layer_name, existing_angle_list, extra_info, point_precision)

        self._length_mid_line = None
        self._width_mid_line = None
        self._angle = None
        angle_list: list[float] = list(self._outline_dict.keys())

        if len(angle_list) != 2 or round(abs(abs(angle_list[0] - angle_list[1]) - 90), 0) > 1:
            raise PolygonAngleException(len(angle_list), self._point_list)

    def is_rect(self) -> bool:
        is_rect = True
        outline_dict: dict[float, dict[float, list[Line]]] = self.get_outline_dict()
        for angle in outline_dict:
            if len(outline_dict[angle]) != 2:
                is_rect = False
        return is_rect

    def get_width(self) -> float:
        point_list: list[list[float]] = self.get_point_list()
        rectangle = ShapelyPolygon(point_list)
        min_bounding_rect = rectangle.minimum_rotated_rectangle

        vertices = min_bounding_rect.exterior.coords
        edge1_length = math.sqrt((vertices[0][0] - vertices[1][0]) ** 2 + (vertices[0][1] - vertices[1][1]) ** 2)
        edge2_length = math.sqrt((vertices[1][0] - vertices[2][0]) ** 2 + (vertices[1][1] - vertices[2][1]) ** 2)

        width = min(edge1_length, edge2_length)
        return round(width)

    def get_width_and_length(self) -> tuple[float, float, float, Optional[Angle]]:
        temp_width = float('inf')
        temp_length = -float('inf')
        length_angle = float('inf')
        angle_file = None
        if sorted(list(self._outline_dict.keys())) == [0.0, 90.0] or \
                (len(list(self._outline_dict.keys())) == 2 and
                 abs(list(self._outline_dict.keys())[0] - list(self._outline_dict.keys())[1]) - 90 < 1):
            for angle, lines in self._outline_dict.items():
                temp_value = lines[next(iter(lines))][0].get_length()
                temp_width = temp_value if temp_width > temp_value else temp_width
                if temp_length < temp_value:
                    temp_length = temp_value
                    length_angle = angle
                    angle_file = list(lines.values())[0][0].get_angle()
            return round(temp_width, 0), round(temp_length, 0), length_angle, angle_file
        else:
            return temp_width, temp_length, length_angle, angle_file

    def get_length_coord_and_width(self) -> str:
        point_list = self._point_list
        mid_line_1: Line = Line((point_list[0][0] + point_list[1][0]) / 2,
                                (point_list[0][1] + point_list[1][1]) / 2,
                                (point_list[2][0] + point_list[3][0]) / 2,
                                (point_list[2][1] + point_list[3][1]) / 2,
                                "", [])

        mid_line_2: Line = Line((point_list[0][0] + point_list[3][0]) / 2,
                                (point_list[0][1] + point_list[3][1]) / 2,
                                (point_list[1][0] + point_list[2][0]) / 2,
                                (point_list[1][1] + point_list[2][1]) / 2,
                                "", [])
        if mid_line_1.get_length() >= mid_line_2.get_length():
            self._angle = mid_line_1.get_angle()
            self._length_mid_line = mid_line_1
            self._width_mid_line = mid_line_2
        else:
            self._angle = mid_line_2.get_angle()
            self._length_mid_line = mid_line_2
            self._width_mid_line = mid_line_1
        return str(round(self._length_mid_line.get_key_coord() + 0.4, 0)) + "_" + str(self.get_width())

    def get_polygon_angle(self) -> Angle:
        outline = self._outline_dict
        max_length = -float('inf')
        the_angle = None
        for angle_value, lines_dict in outline.items():
            for key_coord, lines in lines_dict.items():
                for line in lines:
                    if line.get_length() > max_length:
                        max_length = line.get_length()
                        the_angle = line.get_angle()
        return the_angle

    def update_point_list(self, new_point_list: list[list[float]]):
        self._point_list = new_point_list

    def update_point_list_round(self, point_precision):
        rounded_point_list: list[list[float]] = []
        for coord in self.get_point_list():
            rounded_point_list.append([round(x, point_precision) for x in coord])
        self._point_list = rounded_point_list

    def get_length_mid_line(self) -> Optional[Line]:
        return self._length_mid_line

    def get_width_mid_line(self) -> Optional[Line]:
        return self._width_mid_line

    def get_angle(self) -> Optional[Angle]:
        return self._angle




# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/20
# Author: ChunTsang <zjun5566@163.com>
"""

from dxfgeometry.parser import read_dxf_to_geometry
from dxfgeometry.json_geometry.json_cad_result import JsonCADReadResult, json_cad_read_result_to_object
from dxfgeometry.util.tools import save_json_file


if __name__ == '__main__':
    # Read DXF File
    dxf_file_path = 'demo_file.dxf'
    cad_read_result = read_dxf_to_geometry(dxf_file_path)

    # Export to JSON File
    json_result = JsonCADReadResult(cad_read_result)
    save_json_file(json_result, 'result.json')

    # Reload saved JSON File
    with open('result.json', 'r', encoding='utf-8') as f:
        cad_read_result = json_cad_read_result_to_object(f.read())




# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/19
# Author: ChunTsang <zjun5566@163.com>
"""


class PolygonException(Exception):
    _point_list: list[list[float]]

    def __init__(self, point_list: list[list[float]]):
        super().__init__()
        self._point_list = point_list

    def get_points(self):
        return self._point_list


class PolygonAngleException(PolygonException):
    _angle_num: int

    def __init__(self, angle_num: int, point_list: list[list[float]]):
        super(PolygonException, self).__init__(point_list)
        self._angle_num = angle_num
        self._point_list = point_list

    def __str__(self):
        return f"多边形角度个数错误。有{self._angle_num}个角度，形状点位:{self._point_list}"


class PolygonTinyAreaException(PolygonException):
    def __init__(self, point_list: list[list[float]]):
        super(PolygonTinyAreaException, self).__init__(point_list)

    def __str__(self):
        return "多边形过于小"



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/20
# Author: ChunTsang <zjun5566@163.com>
"""

from json import loads
from src.dxfgeometry.json_geometry.json_base_class import Jsonable
from src.dxfgeometry.json_geometry.json_cad_layer import JsonCADLayer, json_cad_layer_to_object
from src.dxfgeometry.geometry.cad_result import CADReadResult
from src.dxfgeometry.geometry.cad_layer import CADLayer


class JsonCADReadResult(Jsonable):
    file_name: str = None
    layers: dict[str, JsonCADLayer] = None
    x_range: tuple[int, int]
    y_range: tuple[int, int]
    existing_angle_list: list[float]

    def __init__(self, cad_read_result: CADReadResult):
        self.file_name = cad_read_result.get_file_name()
        self.x_range = cad_read_result.get_x_range()
        self.y_range = cad_read_result.get_y_range()
        self.existing_angle_list = cad_read_result.get_existing_angles()

        self.layers = {}
        for layer_name in cad_read_result.get_layers():
            self.layers[layer_name] = JsonCADLayer(cad_read_result.get_layers()[layer_name])


def json_cad_read_result_to_object(json_cad_read_result_str):
    layer_dict: dict[str, CADLayer] = {}
    json_cad_read_result = loads(json_cad_read_result_str)
    existing_angle = json_cad_read_result["existing_angle_list"]
    for layer_name, json_layer in json_cad_read_result["layers"].items():
        layer_dict[layer_name] = json_cad_layer_to_object(json_layer, existing_angle)
    return CADReadResult(json_cad_read_result["file_name"], layer_dict, existing_angle)



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/20
# Author: ChunTsang <zjun5566@163.com>
"""

from src.dxfgeometry.geometry.line import Line
from src.dxfgeometry.geometry.polygon import Polygon
from src.dxfgeometry.geometry.ortho_polygon import OrthoPolygon
from src.dxfgeometry.geometry.text import Text
from src.dxfgeometry.geometry.ortho_hatch import OrthoHatch
from src.dxfgeometry.geometry.hatch import Hatch
from src.dxfgeometry.geometry.point import Point
from src.dxfgeometry.geometry.circle import Circle
from src.dxfgeometry.geometry.arc import Arc
from src.dxfgeometry.geometry.ellipse import Ellipse
from src.dxfgeometry.geometry.polyline import Polyline


class CADLayer:
    _layer_name: str
    _color: str
    _point_list: list[Point]
    _circle_list: list[Circle]
    _arc_list: list[Arc]
    _ellipse_list: list[Ellipse]
    _text_list: list[Text]
    _line_list: list[Line]
    _ortho_polygon_list: list[OrthoPolygon]
    _ortho_hatch_list: list[OrthoHatch]
    _hatch_list: list[Hatch]
    _polyline_list: list[Polyline]
    _polygon_list: list[Polygon]
    _x_range: tuple[float, float] = None
    _y_range: tuple[float, float] = None

    def __init__(self, layer_name: str, color: str):
        self._layer_name = layer_name
        self._color = color
        self._text_list = []
        self._ortho_polygon_list = []
        self._line_list = []
        self._ortho_hatch_list = []
        self._hatch_list = []
        self._point_list = []
        self._circle_list = []
        self._arc_list = []
        self._ellipse_list = []
        self._polyline_list = []
        self._polygon_list = []

    def get_layer_name(self) -> str:
        return self._layer_name

    def get_color(self) -> str:
        return self._color

    def get_text_list(self) -> list[Text]:
        return self._text_list

    def get_line_list(self) -> list[Line]:
        return self._line_list

    def get_polygon_list(self) -> list[Polygon]:
        return self._polygon_list

    def get_ortho_polygon_list(self) -> list[OrthoPolygon]:
        return self._ortho_polygon_list

    def get_ortho_hatch_list(self) -> list[OrthoHatch]:
        return self._ortho_hatch_list

    def get_hatch_list(self) -> list[Hatch]:
        return self._hatch_list

    def get_point_list(self) -> list[Point]:
        return self._point_list

    def get_circle_list(self) -> list[Circle]:
        return self._circle_list

    def get_arc_list(self) -> list[Arc]:
        return self._arc_list

    def get_ellipse_list(self) -> list[Ellipse]:
        return self._ellipse_list

    def get_polyline_list(self) -> list[Polyline]:
        return self._polyline_list

    def add_text(self, text: Text):
        self._text_list.append(text)
        for point in text.get_anchor_point_list():
            self.extend_layer_boundary(point[0], point[1])

    def add_line(self, line: Line):
        self._line_list.append(line)
        self.extend_layer_boundary(line.get_start_point().get_x(), line.get_start_point().get_y())
        self.extend_layer_boundary(line.get_another_point().get_x(), line.get_another_point().get_y())

    def add_polygon(self, polygon: Polygon) -> None:
        self._polygon_list.append(polygon)
        for point in polygon.get_point_list():
            self.extend_layer_boundary(point[0], point[1])

    def add_ortho_polygon(self, polygon: OrthoPolygon):
        self._ortho_polygon_list.append(polygon)
        for point in polygon.get_point_list():
            self.extend_layer_boundary(point[0], point[1])

    def add_hatch(self, hatch: Hatch):
        self._hatch_list.append(hatch)
        for point in hatch.get_point_list():
            self.extend_layer_boundary(point[0], point[1])

    def add_ortho_hatch(self, hatch: OrthoHatch):
        self._ortho_hatch_list.append(hatch)
        for point in hatch.get_point_list():
            self.extend_layer_boundary(point[0], point[1])

    def add_point(self, point: Point):
        self._point_list.append(point)
        self.extend_layer_boundary(point.get_x(), point.get_y())

    def add_circle(self, circle: Circle):
        self._circle_list.append(circle)

    def add_arc(self, arc: Arc):
        self._arc_list.append(arc)

    def add_ellipse(self, ellipse: Ellipse):
        self._ellipse_list.append(ellipse)

    def add_polyline(self, polyline: Polyline):
        self._polyline_list.append(polyline)
        for point in polyline.get_point_list():
            self.extend_layer_boundary(point[0], point[1])

    def get_x_range(self) -> tuple[float, float]:
        return self._x_range

    def get_y_range(self) -> tuple[float, float]:
        return self._y_range

    def extend_layer_boundary(self, x: float, y: float) -> None:
        if self._x_range is None:
            self._x_range = (x, x)
            self._y_range = (y, y)
            return

        (x1, x2) = self._x_range
        (y1, y2) = self._y_range

        if x < x1:
            self._x_range = (x, x2)
        elif x2 < x:
            self._x_range = (x1, x)

        if y < y1:
            self._y_range = (y, y2)
        elif y2 < y:
            self._y_range = (y1, y)



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/20
# Author: ChunTsang <zjun5566@163.com>
"""

from src.dxfgeometry.json_geometry.json_base_class import Jsonable


class JsonCADElement(Jsonable):
    layer_name: str
    type: str

    def __init__(self, layer_name: str, type: str):
        self.layer_name = layer_name
        self.type = type



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/19
# Author: ChunTsang <zjun5566@163.com>
"""

from src.dxfgeometry.geometry.polygon import Polygon
from src.dxfgeometry.config import default_point_precision


class Hatch(Polygon):
    _hatch_pattern_name: str

    def __init__(self, point_list: list[list[float]], layer_name: str, hatch_pattern_name: str,
                 existing_angle_list: list[float], point_precision: int = default_point_precision) -> None:
        super(Hatch, self).__init__(point_list, layer_name, existing_angle_list, point_precision=point_precision)
        self._hatch_pattern_name = hatch_pattern_name

    def get_hatch_pattern_name(self) -> str:
        return self._hatch_pattern_name



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/20
# Author: ChunTsang <zjun5566@163.com>
"""

from src.dxfgeometry.json_geometry.json_cad_element import JsonCADElement
from src.dxfgeometry.geometry.ortho_polygon import OrthoPolygon


class JsonOrthoPolygon(JsonCADElement):
    points: list[float]
    width: float
    length: float
    mid_points: list[float]
    extra_info: dict

    def __init__(self, polygon: OrthoPolygon):
        super(JsonOrthoPolygon, self).__init__(polygon.get_layer_name(), "OrthoPolygon")
        self.points = polygon.get_coord_list_flatten()
        self.width, self.length, _, _ = polygon.get_width_and_length()
        self.extra_info = polygon.get_extra_info()
        self.mid_points = polygon.get_mid_point_list_flatten()


def json_ortho_polygon_to_ortho_polygon(json_ortho_polygon: dict, existing_angles: list[float]) -> OrthoPolygon:
    point_list: list[list[float]] = []
    for i in range(0, len(json_ortho_polygon["points"]), 2):
        point_list.append([json_ortho_polygon["points"][i], json_ortho_polygon["points"][i + 1]])

    if "extra_info" in json_ortho_polygon and json_ortho_polygon["extra_info"] is not None:
        extra_info = json_ortho_polygon.get("extra_info", {})
    else:
        extra_info = {}

    if "layer_name" in json_ortho_polygon and json_ortho_polygon["layer_name"] is not None:
        layer_name = json_ortho_polygon["layer_name"]
    else:
        layer_name = ""

    return OrthoPolygon(point_list, layer_name, existing_angles, extra_info)



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/20
# Author: ChunTsang <zjun5566@163.com>
"""

import json


class Jsonable(object):
    def to_json(self):
        return json.dumps(self, default=lambda o: o.__dict__, indent=4, ensure_ascii=False)

    def save_to_file(self, output):
        return json.dump(self, output, indent=4, default=lambda o: o.__dict__)



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/20
# Author: ChunTsang <zjun5566@163.com>
"""

from src.dxfgeometry.geometry.cad_layer import CADLayer


class CADReadResult:
    _file_name: str = None
    _layers: dict[str, CADLayer] = None
    _x_range: tuple[int|float, int|float]
    _y_range: tuple[int|float, int|float]
    _existing_angle: list[float]

    def __init__(self, file_name: str, layers: dict[str, "CADLayer"], existing_angle: list[float]) -> None:
        self._file_name = file_name
        self._layers = layers
        self._existing_angle = existing_angle

        if len(self._layers) == 0:
            return

        self._x_range = (min([o.get_x_range()[0] for (_, o) in self._layers.items() if o.get_x_range()]),
                         max([o.get_x_range()[1] for (_, o) in self._layers.items() if o.get_x_range()]))
        self._y_range = (min([o.get_y_range()[0] for (_, o) in self._layers.items() if o.get_y_range()]),
                         max([o.get_y_range()[1] for (_, o) in self._layers.items() if o.get_y_range()]))

    def get_file_name(self) -> str:
        return self._file_name

    def get_existing_angles(self) -> list[float]:
        return self._existing_angle

    def get_layers(self) -> dict[str, "CADLayer"]:
        return self._layers

    def get_x_range(self) -> tuple[int, int]:
        return self._x_range

    def get_y_range(self) -> tuple[int, int]:
        return self._y_range

    def merge_with_other_result(self, other: "CADReadResult") -> None:
        self._layers = self._layers | other.get_layers()
        self._x_range = (
            min(self._x_range[0], other.get_x_range()[0]), max(self._x_range[1], other.get_x_range()[1]))
        self._y_range = (
            min(self._y_range[0], other.get_y_range()[0]), max(self._y_range[1], other.get_y_range()[1]))



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/19
# Author: ChunTsang <zjun5566@163.com>
"""

from src.dxfgeometry.geometry.cad_element import CADElement
from src.dxfgeometry.geometry.line import Line
from src.dxfgeometry.config import default_point_precision


def preprocessing_point_list(point_list: list[list[float]], point_precision: int):
    rounded_point_list: list[list[float]] = []
    for coord in point_list:
        rounded_point_list.append([round(x, point_precision) for x in coord])

    points_index_to_remove: list[int] = []
    for i in range(0, len(rounded_point_list) - 1):
        if abs((rounded_point_list[i][0] - rounded_point_list[i + 1][0]) ** 2 +
               (rounded_point_list[i][1] - rounded_point_list[i + 1][1]) ** 2) < 10:
            points_index_to_remove.append(i)

    if abs((rounded_point_list[0][0] - rounded_point_list[-1][0]) ** 2 +
           (rounded_point_list[0][1] - rounded_point_list[-1][1]) ** 2) < 10:
        points_index_to_remove.append(-1)

    for index in sorted(points_index_to_remove, reverse=True):
        del rounded_point_list[index]

    return rounded_point_list


class Polyline(CADElement):
    _point_list: list[list[float]]
    _line_list: list[Line]
    _point_precision: int

    def __init__(self, point_list: list[list[float]], layer_name: str, existing_angle_list: list[float],
                 extra_info=None, point_precision: int = default_point_precision) -> None:
        super(Polyline, self).__init__(layer_name)

        self._point_precision = point_precision
        rounded_point_list = preprocessing_point_list(point_list, self._point_precision)
        self._point_list = rounded_point_list
        self._line_list = []
        extra_info = {} if extra_info is None else extra_info
        self._extra_info = extra_info

        for i in range(len(rounded_point_list) - 1):
            self._line_list.append(Line(rounded_point_list[i][0], rounded_point_list[i][1],
                                        rounded_point_list[i+1][0], rounded_point_list[i+1][1], "",
                                        existing_angle_list))

    def __setitem__(self, key, value):
        self._extra_info[key] = value

    def get_point_list(self) -> list[list[float]]:
        return self._point_list

    def get_x_y_list(self) -> tuple[list[float], list[float]]:
        x_list, y_list = [], []
        for point in self._point_list:
            x_list.append(point[0])
            y_list.append(point[1])
        return x_list, y_list

    def get_line_list(self) -> list[Line]:
        return self._line_list

    def get_coord_list_flatten(self) -> list[float]:
        result = []
        for point in self._point_list:
            result.append(point[0])
            result.append(point[1])
        return result

    def get_extra_info(self):
        return self._extra_info

    def get_point_precision(self) -> int:
        return self._point_precision




# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/20
# Author: ChunTsang <zjun5566@163.com>
"""

from src.dxfgeometry.json_geometry.json_cad_element import JsonCADElement
from src.dxfgeometry.geometry.hatch import Hatch


class JsonHatch(JsonCADElement):
    points: list[float]
    pattern: str

    def __init__(self, hatch: Hatch):
        super(JsonHatch, self).__init__(hatch.get_layer_name(), "HATCH")
        self.points = hatch.get_coord_list_flatten()
        self.pattern = hatch.get_hatch_pattern_name()


def json_hatch_to_hatch(json_hatch: dict, existing_angles: list[float]) -> Hatch:
    point_list: list[list[float]] = []
    for i in range(0, len(json_hatch["points"]), 2):
        point_list.append([json_hatch["points"][i], json_hatch["points"][i + 1]])

    return Hatch(point_list, json_hatch["layer_name"], json_hatch["pattern"], existing_angles)



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/20
# Author: ChunTsang <zjun5566@163.com>
"""

from src.dxfgeometry.json_geometry.json_cad_element import JsonCADElement
from src.dxfgeometry.json_geometry.json_line import JsonLine
from src.dxfgeometry.geometry.polyline import Polyline


class JsonPolyline(JsonCADElement):
    points: list[float]
    line_list: list[JsonLine]
    extra_info: dict

    def __init__(self, polyline: Polyline):
        super(JsonPolyline, self).__init__(polyline.get_layer_name(), "POLYLINE")
        self.points = polyline.get_coord_list_flatten()
        self.line_list = []
        for line in polyline.get_line_list():
            self.line_list.append(JsonLine(line))
        self.extra_info = polyline.get_extra_info()


def json_polyline_to_polyline(json_polyline, existing_angles: list[float]):
    point_list: list[list[float]] = []
    for i in range(0, len(json_polyline["points"]), 2):
        point_list.append([json_polyline["points"][i], json_polyline["points"][i + 1]])

    extra_info = json_polyline["extra_info"] if "extra_info" in json_polyline else {}
    return Polyline(point_list, json_polyline["layer_name"], existing_angles, extra_info)




# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/20
# Author: ChunTsang <zjun5566@163.com>
"""

from src.dxfgeometry.json_geometry.json_cad_element import JsonCADElement
from src.dxfgeometry.geometry.text import Text


class JsonText(JsonCADElement):
    angle: float
    content: str
    anchor_point_list: list[list[float]]
    points: list[float]
    height: float
    width: float

    def __init__(self, text: Text):
        super(JsonText, self).__init__(text.get_layer_name(), "Text")
        self.angle = text.get_angle().get_angle()
        self.content = text.get_content()
        self.anchor_point_list = text.get_anchor_point_list()
        self.width = text.get_width()
        self.height = text.get_height()
        self.points = []
        for point in text.get_anchor_point_list():
            self.points.append(point[0])
            self.points.append(point[1])


def json_text_to_text(json_text, existing_angle_list: list[float]) -> Text:
    width = json_text["width"] if "width" in json_text and json_text["width"] is not None else 0
    height = json_text["height"] if "height" in json_text and json_text["height"] is not None else 0
    return Text(json_text["anchor_point_list"], json_text["layer_name"], json_text["content"], json_text["angle"],
                width, height, existing_angle_list)




# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/20
# Author: ChunTsang <zjun5566@163.com>
"""

from src.dxfgeometry.json_geometry.json_base_class import Jsonable
from src.dxfgeometry.json_geometry.json_point import JsonPoint, json_point_to_point
from src.dxfgeometry.json_geometry.json_circle import JsonCircle, json_circle_to_circle
from src.dxfgeometry.json_geometry.json_arc import JsonArc, json_arc_to_arc
from src.dxfgeometry.json_geometry.json_ellipse import JsonEllipse, json_ellipse_to_ellipse
from src.dxfgeometry.json_geometry.json_text import JsonText, json_text_to_text
from src.dxfgeometry.json_geometry.json_line import JsonLine, json_line_to_line
from src.dxfgeometry.json_geometry.json_ortho_polygon import JsonOrthoPolygon, json_ortho_polygon_to_ortho_polygon
from src.dxfgeometry.json_geometry.json_ortho_hatch import JsonOrthoHatch, json_ortho_hatch_to_ortho_hatch
from src.dxfgeometry.json_geometry.json_hatch import JsonHatch, json_hatch_to_hatch
from src.dxfgeometry.json_geometry.json_polyline import JsonPolyline, json_polyline_to_polyline
from src.dxfgeometry.json_geometry.json_polygon import JsonPolygon, json_polygon_to_polygon
from src.dxfgeometry.geometry.cad_layer import CADLayer


class JsonCADLayer(Jsonable):
    layer_name: str
    color: str
    point_list: list[JsonPoint]
    circle_list: list[JsonCircle]
    arc_list: list[JsonArc]
    ellipse_list: list[JsonEllipse]
    text_list: list[JsonText]
    line_list: list[JsonLine]
    ortho_polygon_list: list[JsonOrthoPolygon]
    ortho_hatch_list: list[JsonOrthoHatch]
    hatch_list: list[JsonHatch]
    polyline_list: list[JsonPolyline]
    polygon_list: list[JsonPolygon]
    x_range: tuple[float, float]
    y_range: tuple[float, float]

    def __init__(self, cad_layer: CADLayer):
        self.layer_name = cad_layer.get_layer_name()
        self.color = cad_layer.get_color()

        self.point_list = []
        for point in cad_layer.get_point_list():
            self.point_list.append(JsonPoint(point))

        self.circle_list = []
        for circle in cad_layer.get_circle_list():
            self.circle_list.append(JsonCircle(circle))

        self.arc_list = []
        for arc in cad_layer.get_arc_list():
            self.arc_list.append(JsonArc(arc))

        self.ellipse_list = []
        for ellipse in cad_layer.get_ellipse_list():
            self.ellipse_list.append(JsonEllipse(ellipse))

        self.text_list = []
        for text in cad_layer.get_text_list():
            self.text_list.append(JsonText(text))

        self.line_list = []
        for line in cad_layer.get_line_list():
            self.line_list.append(JsonLine(line))

        self.ortho_polygon_list = []
        for ortho_polygon in cad_layer.get_ortho_polygon_list():
            self.ortho_polygon_list.append(JsonOrthoPolygon(ortho_polygon))

        self.ortho_hatch_list = []
        for ortho_hatch in cad_layer.get_ortho_hatch_list():
            self.ortho_hatch_list.append(JsonOrthoHatch(ortho_hatch))

        self.hatch_list = []
        for hatch in cad_layer.get_hatch_list():
            self.hatch_list.append(JsonHatch(hatch))

        self.polyline_list = []
        for polyline in cad_layer.get_polyline_list():
            self.polyline_list.append(JsonPolyline(polyline))

        self.polygon_list = []
        for polygon in cad_layer.get_polygon_list():
            self.polygon_list.append(JsonPolygon(polygon))

        self.x_range = cad_layer.get_x_range()
        self.y_range = cad_layer.get_y_range()


def json_cad_layer_to_object(json_cad_layer, existing_angle: list[float]):
    layer_name = json_cad_layer["layer_name"]
    color = json_cad_layer["color"]
    cad_layer = CADLayer(layer_name, color)

    for json_point in json_cad_layer["point_list"]:
        cad_layer.add_point(json_point_to_point(json_point))

    for json_circle in json_cad_layer["circle_list"]:
        cad_layer.add_circle(json_circle_to_circle(json_circle))

    for json_arc in json_cad_layer["arc_list"]:
        cad_layer.add_arc(json_arc_to_arc(json_arc))

    for json_ellipse in json_cad_layer["ellipse_list"]:
        cad_layer.add_ellipse(json_ellipse_to_ellipse(json_ellipse))

    for json_text in json_cad_layer["text_list"]:
        cad_layer.add_text(json_text_to_text(json_text, existing_angle))

    for json_line in json_cad_layer["line_list"]:
        cad_layer.add_line(json_line_to_line(json_line, existing_angle))

    for json_ortho_polygon in json_cad_layer["ortho_polygon_list"]:
        cad_layer.add_ortho_polygon(json_ortho_polygon_to_ortho_polygon(json_ortho_polygon, existing_angle))

    for json_ortho_hatch in json_cad_layer["ortho_hatch_list"]:
        cad_layer.add_ortho_hatch(json_ortho_hatch_to_ortho_hatch(json_ortho_hatch, existing_angle))

    for json_hatch in json_cad_layer["hatch_list"]:
        cad_layer.add_hatch(json_hatch_to_hatch(json_hatch, existing_angle))

    for json_polyline in json_cad_layer["polyline_list"]:
        cad_layer.add_polyline(json_polyline_to_polyline(json_polyline, existing_angle))

    for json_polygon in json_cad_layer["polygon_list"]:
        cad_layer.add_polygon(json_polygon_to_polygon(json_polygon, existing_angle))

    return cad_layer




# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/19
# Author: ChunTsang <zjun5566@163.com>
"""

import ezdxf
import math
import logging
from shapely.geometry import LineString as ShapelyLineString
from util.color import get_layer_color
from geometry.arc import Arc
from geometry.ellipse import Ellipse
from geometry.circle import Circle
from geometry.text import Text
from geometry.hatch import Hatch
from geometry.ortho_hatch import OrthoHatch
from geometry.line import Line
from geometry.point import Point
from geometry.cad_layer import CADLayer
from geometry.polyline import Polyline
from geometry.polygon import Polygon
from geometry.ortho_polygon import OrthoPolygon
from geometry.cad_result import CADReadResult
from exception.reader_exception import ReadElementException, handle_exception
from exception.polygon_exception import PolygonAngleException, PolygonTinyAreaException
from json_geometry.json_cad_layer import JsonCADLayer


def read_dxf_to_geometry(dxf_file_path: str) -> CADReadResult:
    doc = ezdxf.readfile(dxf_file_path)
    layers_color = get_layer_color(doc)
    msp = doc.modelspace()

    group: dict = msp.groupby(dxfattrib='layer')
    layers_to_read = []
    existing_angle_list = []
    layer_dict: dict[str, CADLayer] = {}
    element_with_exception: dict[str, list[ReadElementException]] = {}

    for layer_name in list(group.keys()):
        if layer_name in group:
            layers_to_read.append((layer_name, group[layer_name]))

    while len(layers_to_read) > 0:
        next_layer_name, next_layer = layers_to_read[0]
        color = layers_color[next_layer_name]
        new_layer_list = parse_layer(next_layer, next_layer_name, layer_dict, existing_angle_list,
                                     element_with_exception, color)
        layers_to_read.pop(0)
        layers_to_read += new_layer_list

    return CADReadResult(dxf_file_path, layer_dict, existing_angle_list)


def parse_layer(layer, layer_name: str, layer_dict: dict[str, CADLayer], existing_angle_list: list[float],
                element_with_exception: dict[str, list[ReadElementException]], color: str):
    layer_dict.setdefault(layer_name, CADLayer(layer_name, color))
    element_with_exception.setdefault(layer_name, [])

    new_layer_list: list[tuple[str, CADLayer]] = []
    for new_element in layer:
        if new_element.dxftype() == 'LWPOLYLINE' or new_element.dxftype() == 'POLYLINE':
            authentic_is_closed = new_element.is_closed
            point_list = list(new_element.vertices_in_wcs())
            if (point_list[0][0] - point_list[-1][0]) ** 2 + (point_list[0][1] - point_list[-1][1]) ** 2 < 10:
                authentic_is_closed = True
            parse_polyline(point_list, layer_dict, layer_name, existing_angle_list, element_with_exception,
                           authentic_is_closed)

        elif new_element.dxftype() == 'LINE':
            try:
                layer_dict.get(layer_name).add_line(Line(new_element.dxf.start[0], new_element.dxf.start[1],
                                                    new_element.dxf.end[0], new_element.dxf.end[1],
                                                    layer_name, existing_angle_list))
            except ValueError as e:
                handle_exception(e, [[new_element.dxf.start[0], new_element.dxf.start[1]],
                                     [new_element.dxf.end[0], new_element.dxf.end[1]]],
                                 element_with_exception[layer_name])
                continue

        elif new_element.dxftype() == 'POINT':
            layer_dict.get(layer_name).add_point(Point(new_element.dxf.location[0], new_element.dxf.location[1],
                                                       layer_name))

        elif new_element.dxftype() == 'HATCH':
            for path in new_element.paths.paths:
                point_list: list[list[float]] = []
                if isinstance(path, ezdxf.entities.PolylinePath):
                    for vertices in path.vertices:
                        point_list.append([vertices[0], vertices[1]])
                elif isinstance(path, ezdxf.entities.EdgePath):
                    edge_list = path.edges
                    for edge in edge_list:
                        if isinstance(edge, ezdxf.entities.LineEdge):
                            x, y = edge.start
                            point_list.append([x, y])
                        else:
                            logging.warning("暂不支持: %s", type(edge))
                            continue
                if len(point_list) == 0:
                    continue
                try:
                    layer_dict.get(layer_name).add_ortho_hatch(
                        OrthoHatch(point_list, layer_name, new_element.dxf.pattern_name, existing_angle_list))
                except PolygonAngleException:
                    try:
                        hatch = Hatch(point_list, layer_name, new_element.dxf.pattern_name, existing_angle_list)
                        layer_dict.get(layer_name).add_hatch(hatch)
                    except Exception as e:
                        handle_exception(e, point_list, element_with_exception[layer_name])
                        continue
                except Exception as e:
                    handle_exception(e, point_list, element_with_exception[layer_name])
                    continue

        elif new_element.dxftype() == 'TEXT' or new_element.dxftype() == 'MTEXT':
            anchor_point_list: list[list[float]] = [[new_element.dxf.insert[0], new_element.dxf.insert[1]]]
            if new_element.dxftype() == 'TEXT':
                content_text: str = new_element.dxf.text
                if new_element.dxf.align_point is not None:
                    anchor_point_list.append([new_element.dxf.align_point[0], new_element.dxf.align_point[1]])
                angle: float = - round(new_element.dxf.rotation + 90, 10)
                while angle >= 180:
                    angle = angle - 180
                while angle < 0:
                    angle = angle + 180
                width = max(new_element.dxf.width, 0.1)
                text = Text(anchor_point_list, layer_name, content_text, angle, width,
                            new_element.dxf.height, existing_angle_list)
                layer_dict.get(layer_name).add_text(text)

            elif new_element.dxftype() == 'MTEXT':
                content_text: str = new_element.plain_text()
                text_x = new_element.dxf.text_direction[0]
                text_y = new_element.dxf.text_direction[1]
                cos_theta = text_x / ((text_x ** 2 + text_y ** 2) ** 0.5)
                if abs(abs(cos_theta) - 1) < 0.01:
                    text = Text(anchor_point_list, layer_name, content_text, 90, new_element.dxf.width,
                                new_element.dxf.char_height, existing_angle_list)
                else:
                    angle = round(
                        math.atan(new_element.dxf.text_direction[0] / new_element.dxf.text_direction[1]) * 57.2958, 10)
                    while angle >= 180:
                        angle = angle - 180
                    while angle < 0:
                        angle = angle + 180
                    width = max(new_element.dxf.width, 0.1)
                    text = Text(anchor_point_list, layer_name, content_text, angle, width,
                                new_element.dxf.char_height, existing_angle_list)
                layer_dict.get(layer_name).add_text(text)

        elif new_element.dxftype() == 'INSERT':
            try:
                new_msp = new_element.explode().groupby('layer')
            except Exception as _:
                continue
            for new_layer_name in new_msp.keys():
                new_layer_list.append((new_layer_name, new_msp[new_layer_name]))

        elif new_element.dxftype() == "DIMENSION":
            try:
                new_msp = new_element.explode().groupby('layer')
            except Exception as e:
                continue
            for new_layer_name in new_msp.keys():
                new_layer_list.append((layer_name, new_msp[new_layer_name]))

        elif new_element.dxftype() == "CIRCLE":
            coord_x: int = new_element.dxf.center[0]
            coord_y: int = new_element.dxf.center[1]
            radius: int = new_element.dxf.radius
            layer_dict.get(layer_name).add_circle(Circle(coord_x, coord_y, radius, layer_name))

        elif new_element.dxftype() == "ARC":
            center_x: float = new_element.dxf.center[0]
            center_y: float = new_element.dxf.center[1]
            radius: float = new_element.dxf.radius
            start_angle: float = new_element.dxf.start_angle
            end_angle: float = new_element.dxf.end_angle
            layer_dict.get(layer_name).add_arc(Arc(center_x, center_y, start_angle, end_angle, radius, layer_name))

        elif new_element.dxftype() == "ELLIPSE":
            center_x: float = new_element.dxf.center[0]
            center_y: float = new_element.dxf.center[1]
            major_axis = new_element.dxf.major_axis
            major_angle = get_line_angle(ShapelyLineString([(0, 0), (major_axis[0], major_axis[1])]))
            ratio: float = new_element.dxf.ratio

            start_param: float = new_element.dxf.start_param
            end_param: float = new_element.dxf.end_param

            layer_dict.get(layer_name).add_ellipse(Ellipse(center_x, center_y, start_param, end_param,
                                                           major_axis, ratio, layer_name))
            equal_radius = math.sqrt(ratio * major_axis.magnitude * major_axis.magnitude)
            if new_element.dxf.extrusion[2] >= 0:
                start_angle: float = start_param / math.pi * 180 + major_angle
                end_angle: float = end_param / math.pi * 180 + major_angle
            else:
                start_angle: float = major_angle - end_param / math.pi * 180
                end_angle: float = major_angle - start_param / math.pi * 180
            if start_angle <= 0:
                start_angle += 360
            if end_angle <= 0:
                end_angle += 360

            layer_dict.get(layer_name).add_arc(Arc(center_x, center_y, start_angle, end_angle, equal_radius,
                                                   layer_name))

        else:
            logging.warning(f"parse_layer(): 暂不支持该格式: {new_element.dxftype()} {layer_name}")

    return new_layer_list


def parse_polyline(point_list: list[list[float]], layer_dict: dict[str, CADLayer], layer_name: str,
                   existing_angle_list: list[float], element_with_exception: dict[str, list[ReadElementException]],
                   is_enclosed: bool):
    if is_enclosed:
        try:
            layer_dict.get(layer_name).add_ortho_polygon(OrthoPolygon(point_list, layer_name, existing_angle_list))
        except PolygonTinyAreaException as e:
            handle_exception(e, point_list, element_with_exception[layer_name])
            return
        except PolygonAngleException as e:
            layer_dict.get(layer_name).add_polygon(Polygon(point_list, layer_name, existing_angle_list))
        except Exception as e:
            handle_exception(e, point_list, element_with_exception[layer_name])
            return
    else:
        if len(point_list) == 2:
            try:
                layer_dict.get(layer_name).add_line(Line(point_list[0][0], point_list[0][1],
                                                         point_list[1][0], point_list[1][1],
                                                         layer_name, existing_angle_list))
            except Exception as e:
                handle_exception(e, point_list, element_with_exception[layer_name])
                return
        else:
            polyline_point_list: list[list[float]] = []
            for point in point_list:
                polyline_point_list.append([point[0], point[1]])
            try:
                layer_dict.get(layer_name).add_polyline(Polyline(polyline_point_list, layer_name, existing_angle_list))
            except Exception as e:
                handle_exception(e, polyline_point_list, element_with_exception[layer_name])
                return


def get_line_angle(line: ShapelyLineString):
    x1, y1 = line.coords[0]
    x2, y2 = line.coords[-1]
    dx = x2 - x1
    dy = y2 - y1
    angle = math.atan2(dy, dx)
    return math.degrees(angle)




# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/19
# Author: ChunTsang <zjun5566@163.com>
"""

from src.dxfgeometry.geometry.cad_element import CADElement
import math
from src.dxfgeometry.config import default_point_precision


class Ellipse(CADElement):
    _x: float
    _y: float
    _start_param: float
    _end_param: float
    _major_axis: any
    _ratio: float
    _radius: float
    _xfact: float
    _yfact: float
    _point_precision: int

    def __init__(self, x: float, y: float, start_param: float, end_param: float, major_axis: any,
                 ratio: float, layer_name: str, point_precision: int = default_point_precision) -> None:
        super(Ellipse, self).__init__(layer_name)

        self._point_precision = point_precision
        self._x = round(x, self._point_precision)
        self._y = round(y, self._point_precision)
        self._major_axis = major_axis
        self._start_param = round(start_param, self._point_precision)
        self._end_param = round(end_param, self._point_precision)
        self._ratio = round(ratio, self._point_precision)
        self._angle1 = self._end_param * 2 * math.pi
        self._angle2 = self._start_param * 2 * math.pi
        self._rx, self._ry = self._major_axis.magnitude, self._major_axis.magnitude * self._ratio
        self._radius = math.sqrt(self._major_axis.x ** 2 + self._major_axis.y ** 2)
        self.xfact = self._rx / self._radius
        self.yfact = self._ry / self._radius

    def get_x(self) -> float:
        return self._x

    def get_y(self) -> float:
        return self._y

    def get_start_param(self) -> float:
        return self._start_param

    def get_end_param(self) -> float:
        return self._end_param

    def get_major_axis(self):
        return self._major_axis

    def get_ratio(self) -> float:
        return self._ratio

    def get_radius(self) -> float:
        return self._radius

    def get_xfact(self) -> float:
        return self.xfact

    def get_yfact(self) -> float:
        return self.yfact

    def get_point_precision(self) -> int:
        return self._point_precision

    def get_start_point(self, angle_distance) -> tuple[float, float]:
        tran_angle_distance = angle_distance / 180 * math.pi
        x1 = self._x + self._rx * math.cos(self._angle1 - tran_angle_distance)
        y1 = self._y + self._ry * math.sin(self._angle1 - tran_angle_distance)
        return x1, y1

    def get_end_point(self, angle_distance) -> tuple[float, float]:
        tran_angle_distance = angle_distance / 180 * math.pi
        return self._x + self._rx * math.sin(self._angle2 + tran_angle_distance), self._y + self._ry * math.sin \
            (self._angle2 + tran_angle_distance)



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/19
# Author: ChunTsang <zjun5566@163.com>
"""

default_point_precision = 4
default_angle_precision = 4
default_min_angle_difference = 0.25
default_strong_orthogonality = True
default_expanded_length = 10000000000
default_point_min_diff = 0.1
default_angle_threshold = 5
default_coord_key_precision = -1
default_minimum_point_distance = 10


# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/20
# Author: ChunTsang <zjun5566@163.com>
"""


class ReadElementException(Exception):
    _point_list: list[list[float]]
    _reason: str

    def __init__(self, point_list: list[list[float]], reason: str):
        self._point_list = point_list
        self._reason = reason

    def get_point_list(self) -> list[list[float]]:
        return self._point_list

    def get_reason(self) -> str:
        return self._reason

    def __str__(self):
        return self._reason


def handle_exception(e: Exception, point_list: list[list[float]], shape_with_exception: list[ReadElementException]):
    shape_with_exception.append(ReadElementException(point_list, str(e)))



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/20
# Author: ChunTsang <zjun5566@163.com>
"""

from src.dxfgeometry.json_geometry.json_cad_element import JsonCADElement
from src.dxfgeometry.geometry.ellipse import Ellipse
from ezdxf.math import Vec3


class JsonEllipse(JsonCADElement):
    x: float
    y: float
    start_param: float
    end_param: float
    major_axis: any
    ratio: float
    radius: float
    xfact: float
    yfact: float

    def __init__(self, ellipse: Ellipse):
        super(JsonEllipse, self).__init__(ellipse.get_layer_name(), 'Ellipse')
        self.x = ellipse.get_x()
        self.y = ellipse.get_y()
        self.start_param = ellipse.get_start_param()
        self.end_param = ellipse.get_end_param()
        self.major_axis = [ellipse.get_major_axis().x, ellipse.get_major_axis().y, ellipse.get_major_axis().z]
        self.ratio = ellipse.get_ratio()
        self.radius = ellipse.get_radius()
        self.xfact = ellipse.get_xfact()
        self.yfact = ellipse.get_yfact()
        self.electric_info = {}

    def __setitem__(self, key, value):
        self.electric_info[key] = value


def json_ellipse_to_ellipse(json_ellipse: dict) -> Ellipse:
    v = Vec3(json_ellipse['major_axis'][0], json_ellipse['major_axis'][1], json_ellipse['major_axis'][2])
    return Ellipse(json_ellipse['x'], json_ellipse['y'], json_ellipse['start_param'], json_ellipse['end_param'],
                   v, json_ellipse['ratio'], json_ellipse['layer_name'])



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/20
# Author: ChunTsang <zjun5566@163.com>
"""

from src.dxfgeometry.json_geometry.json_cad_element import JsonCADElement
from src.dxfgeometry.geometry.line import Line


class JsonLine(JsonCADElement):
    points: list[float]
    start_x: float
    start_y: float
    length: float
    extra_info: dict

    def __init__(self, line: Line):
        super(JsonLine, self).__init__(line.get_layer_name(), "Line")
        self.points = []
        for point in line.get_ori_points():
            self.points.append(point[0])
            self.points.append(point[1])
        self.start_x = line.get_start_point().get_x()
        self.start_y = line.get_start_point().get_y()
        self.length = line.get_length()
        self.extra_info = line.get_extra_info()


def json_line_to_line(json_line, existing_angle_list: list[float]) -> Line:
    extra_info = json_line["extra_info"] if "extra_info" in json_line else {}
    return Line(json_line["points"][0], json_line["points"][1], json_line["points"][2], json_line["points"][3],
                json_line["layer_name"], existing_angle_list, extra_info)




# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/19
# Author: ChunTsang <zjun5566@163.com>
"""


class CADElement(object):
    _layer_name: str = None

    def __init__(self, layer_name: str) -> None:
        self._layer_name = layer_name

    def get_layer_name(self) -> str:
        return self._layer_name

    def set_layer_name(self, layer_name: str) -> None:
        self._layer_name = layer_name



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/19
# Author: ChunTsang <zjun5566@163.com>
"""

from src.dxfgeometry.geometry.cad_element import CADElement
from src.dxfgeometry.config import default_point_precision


class Circle(CADElement):
    _x: float
    _y: float
    _radius: float
    _point_precision: int

    def __init__(self, x: float, y: float, radius: float, layer_name: str,
                 point_precision: int = default_point_precision) -> None:
        super(Circle, self).__init__(layer_name)

        self._point_precision = point_precision
        self._x = round(x, self._point_precision)
        self._y = round(y, self._point_precision)
        self._radius = radius

    def get_x(self) -> float:
        return self._x

    def get_y(self) -> float:
        return self._y

    def get_radius(self) -> float:
        return self._radius

    def get_point_precision(self) -> int:
        return self._point_precision



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/19
# Author: ChunTsang <zjun5566@163.com>
"""
import math
from typing import Optional
from shapely.geometry import Polygon as ShapelyPolygon
import numpy as np
from src.dxfgeometry.config import default_point_precision, default_angle_precision, default_min_angle_difference
from src.dxfgeometry.json_geometry.json_base_class import Jsonable


def calculate_distance(point1: Optional[list], point2: list):
    if point1 is None:
        return float('inf')
    x1, y1 = point1[0], point1[1]
    x2, y2 = point2[0], point2[1]
    distance = math.sqrt(math.pow(x2 - x1, 2) + math.pow(y2 - y1, 2))
    return round(distance, default_point_precision)


def simplify_polygon_vertices(points):
    poly = ShapelyPolygon(points)
    boundary = poly.exterior
    coords = list(boundary.coords)
    coords = coords[:-1]

    vertices = []
    for i in range(len(coords)):
        prev = coords[i - 1]
        curr = coords[i]
        next_point = coords[(i + 1) % len(coords)]
        vector1 = np.array([curr[0] - prev[0], curr[1] - prev[1]])
        vector2 = np.array([next_point[0] - curr[0], next_point[1] - curr[1]])
        if not is_collinear(vector1, vector2):
            vertices.append(list(curr))
    return vertices


def is_collinear(vector1, vector2):
    cross_product = np.cross(vector1, vector2)
    return abs(cross_product) < 1e-10


def get_angle(x1, y1, x2, y2) -> float:
    x1, y1 = round(x1, default_point_precision), round(y1, default_point_precision)
    x2, y2 = round(x2, default_point_precision), round(y2, default_point_precision)
    if x1 == x2 and y1 == y2:
        raise ValueError("Line初始化错误，两个点坐标相同", x1, x2)

    if x1 > x2:
        x1, y1, x2, y2 = x2, y2, x1, y1
    if x1 == x2:
        return 0.0
    else:
        k = (y2 - y1) / (x2 - x1)
        angle = 90.0 - math.atan(k) * 180.0 / math.pi
        if abs(angle - 180) < default_min_angle_difference:
            angle = 0.0
        return round(angle, default_angle_precision)


def get_rect_mid_point(point_list: list[list[float]]):
    return [calculate_mid_point(point_list[-1 + i], point_list[i]) for i in range(len(point_list))]


def calculate_mid_point(point1: list[float], point2: list[float]):
    x1, y1 = point1[0], point1[1]
    x2, y2 = point2[0], point2[1]
    return [(x1 + x2) / 2, (y1 + y2) / 2]


def save_json_file(json_object: Jsonable, json_file_dir: str):
    with open(json_file_dir, 'w') as f:
        json_object.save_to_file(f)
    print(f">>>>>> Saved in '{json_file_dir}' <<<<<<")



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/19
# Author: ChunTsang <zjun5566@163.com>
"""


class LineLengthException(ValueError):
    _point_list: list[list[float]]
    _track_back: str

    def __init__(self, point_list: list[list[float]]):
        self._point_list = point_list
        self._track_back = ""

    def __str__(self):
        return "该直线长度为0"

    def append_track_back(self, trace_back: str) -> None:
        self._track_back += trace_back

    def get_point_list(self) -> list[list[float]]:
        return self._point_list



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/19
# Author: ChunTsang <zjun5566@163.com>
"""

from src.dxfgeometry.geometry.angle import Angle
from src.dxfgeometry.geometry.cad_element import CADElement


class Text(CADElement):
    _angle: Angle
    _content: str
    _width: float
    _height: float
    _anchor_point_list: list[list[float]]
    _layer_name: str

    def __init__(self, anchor_point_list: list[list[float]], layer_name: str, content: str,
                 angle: float, width: float, height: float, existing_angle_list: list[float]) -> None:
        super(Text, self).__init__(layer_name)

        self._layer_name = layer_name
        self._anchor_point_list = []
        for point in anchor_point_list:
            self.add_anchor_point(point)
        self._angle = Angle(angle, existing_angle_list)
        self._content = content
        self._width = max(width, 0.1) if width is not None else 0.1
        self._height = height

    def __repr__(self):
        return self._content

    def add_anchor_point(self, point):
        if len(self._anchor_point_list) == 0:
            self._anchor_point_list.append(point)
            return
        for existing_point in self._anchor_point_list:
            if abs(point[0] - existing_point[0]) > 10 or abs(point[1] - existing_point[1]) > 10:
                self._anchor_point_list.append(point)
                return

    def get_angle(self) -> Angle:
        return self._angle

    def get_content(self) -> str:
        return self._content

    def update_content(self, new_content):
        self._content = new_content

    def get_width(self) -> float:
        return self._width

    def get_height(self) -> float:
        return self._height

    def get_anchor_point_list(self) -> list[list[float]]:
        return self._anchor_point_list



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/19
# Author: ChunTsang <zjun5566@163.com>
"""

import math
from src.dxfgeometry.geometry.point import Point
from src.dxfgeometry.geometry.angle import Angle
from src.dxfgeometry.geometry.cad_element import CADElement
from src.dxfgeometry.config import default_point_precision
from src.dxfgeometry.exception.line_exception import LineLengthException


class Line(CADElement):
    _k: float
    _start_point: Point
    _another_point: Point
    _angle: Angle
    _length: float
    _key_coord: float
    _length_coord: float
    _ori_points: list[list[float]]
    _point_precision: int

    def __init__(self, x1: float, y1: float, x2: float, y2: float, layer_name: str, existing_angle_list: list[float],
                 extra_info=None, point_precision: int = default_point_precision) -> None:
        super(Line, self).__init__(layer_name)
        self._point_precision = point_precision
        x1, y1 = round(x1, self._point_precision), round(y1, self._point_precision)
        x2, y2 = round(x2, self._point_precision), round(y2, self._point_precision)

        extra_info = {} if extra_info is None else extra_info
        self._extra_info = extra_info

        if math.sqrt((x1 - x2)**2 + (y1 - y2)**2) < 1:
            raise LineLengthException([[x1, y1], [x2, y2]])

        if x1 > x2:
            x1, y1, x2, y2 = x2, y2, x1, y1

        if x1 == x2:
            self._k = math.inf
            self._angle = Angle(0.0, existing_angle_list)
            if abs(0 - self._angle.get_angle()) > 45:
                self._start_point = Point(x1, max(y1, y2), layer_name)
                self._another_point = Point(x1, min(y1, y2), layer_name)
            else:
                self._start_point = Point(x1, min(y1, y2), layer_name)
                self._another_point = Point(x1, max(y1, y2), layer_name)
            self._length = abs(y1 - y2)
        else:
            self._k = (y2 - y1) / (x2 - x1)
            angle = 90.0 - math.atan(self._k) * 180.0 / math.pi
            self._angle = Angle(angle, existing_angle_list)
            if abs(angle - self._angle.get_angle()) > 45:
                x1, y1, x2, y2 = x2, y2, x1, y1
            self._start_point = Point(x1, y1, layer_name)
            self._another_point = Point(x2, y2, layer_name)
            self._length = math.sqrt((x1 - x2) ** 2 + (y1 - y2) ** 2)

        self._ori_points = [[x1, y1], [x2, y2]]

        self._key_coord, self._length_coord = self._angle.get_key_coord_len_coord_of_point(self._start_point)

    def __setitem__(self, key, value):
        self._extra_info[key] = value

    def __repr__(self) -> str:
        return "*线条* 起始点: (" + str(self.get_start_point().get_x()) + ", " + str(self.get_start_point().get_y()) \
               + ") 长度: " + str(self._length) + " 角度: " + str(self._angle.get_angle()) + " length coord " \
               + str(self._length_coord) + " key " + str(self._key_coord)

    def equal(self, other: "Line", precision: float) -> bool:
        if other is None:
            return False
        return self._start_point.equal(other.get_start_point(), precision) and \
               abs(self._length - other.get_length()) < precision and \
               abs(self._angle.get_angle() - other.get_angle().get_angle()) < precision

    def expend_at_both_end(self, length: float) -> None:
        self._start_point = self._angle.get_point_at_dist(self._start_point, length, True)
        self._another_point = self._angle.get_point_at_dist(self._another_point, length, False)
        self._length = self._length + 2 * length
        self._length_coord = self._length_coord - length
        self._ori_points = [self.get_start_point().get_x_y(), self.get_another_point().get_x_y()]

    def shorten_at_both_end(self, length: float) -> None:
        self._start_point = self._angle.get_point_at_dist(self._start_point, length, False)
        self._another_point = self._angle.get_point_at_dist(self._another_point, length, True)
        self._length = self._length - 2 * length
        self._length_coord = self._length_coord + length
        self._ori_points = [self.get_start_point().get_x_y(), self.get_another_point().get_x_y()]

    def shorten_at_start(self, length: float) -> None:
        self._start_point = self._angle.get_point_at_dist(self._start_point, length, False)
        self._length = self._length - length
        self._length_coord = self._length_coord + length
        self._ori_points = [self.get_start_point().get_x_y(), self.get_another_point().get_x_y()]

    def expand_at_start(self, length: float) -> None:
        self._start_point = self._angle.get_point_at_dist(self._start_point, length, True)
        self._length = self._length + length
        self._length_coord = self._length_coord - length
        self._ori_points = [self.get_start_point().get_x_y(), self.get_another_point().get_x_y()]

    def expand_at_end(self, length: float) -> None:
        self._length = self._length + length
        self._another_point = self._angle.get_point_at_dist(self._another_point, length, False)
        self._ori_points = [self.get_start_point().get_x_y(), self.get_another_point().get_x_y()]

    def shorten_at_end(self, length: float) -> None:
        self._length = self._length - length
        self._another_point = self._angle.get_point_at_dist(self._another_point, length, True)
        self._ori_points = [self.get_start_point().get_x_y(), self.get_another_point().get_x_y()]

    def get_mid_point(self) -> Point:
        return self._angle.get_mid_point(self._start_point, self._length)

    def get_x_and_y_list(self) -> tuple[list[float], list[float]]:
        x_list: list[float] = [self._start_point.get_x(), self.get_another_point().get_x()]
        y_list: list[float] = [self._start_point.get_y(), self.get_another_point().get_y()]
        return x_list, y_list

    def get_ori_points(self) -> list[list[float]]:
        return self._ori_points

    def get_start_point(self) -> Point:
        return self._start_point

    def get_length(self) -> float:
        return self._length

    def set_length(self, length: float):
        self._length = length

    def get_angle(self) -> Angle:
        return self._angle

    def get_another_point(self) -> Point:
        return self._another_point

    def get_key_coord(self) -> float:
        return self._key_coord

    def get_length_coord(self) -> float:
        return self._length_coord

    def get_mapping_length_coord(self, mapping_angle) -> float:
        _, len_coord = mapping_angle.get_key_coord_len_coord_of_point(self._start_point)
        return len_coord

    def get_end_length_coord(self) -> float:
        return self._length_coord + self._length

    def get_mid_length_coord(self) -> float:
        return self._length_coord + self._length * 0.5

    def get_extra_info(self):
        return self._extra_info

    def get_k(self) -> float:
        return self._k

    def get_point_precision(self) -> int:
        return self._point_precision



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/19
# Author: ChunTsang <zjun5566@163.com>
"""

from typing import Optional
from src.dxfgeometry.geometry.angle import Angle
from src.dxfgeometry.geometry.line import Line
from src.dxfgeometry.geometry.cad_element import CADElement
from src.dxfgeometry.config import default_point_precision, default_angle_threshold, default_coord_key_precision, \
    default_minimum_point_distance
from src.dxfgeometry.util.tools import calculate_distance, simplify_polygon_vertices, get_angle, get_rect_mid_point
from src.dxfgeometry.exception.line_exception import LineLengthException
from src.dxfgeometry.exception.polygon_exception import PolygonTinyAreaException


class Polygon(CADElement):
    _point_list: list[list[float]] = []
    _mid_point_list: list[list[float]] = []
    _outline_dict: dict[float, dict[float, list[Line]]] = {}
    _point_precision: int
    _extra_info: dict

    def __init__(self, point_list: list[list[float]], layer_name: str, existing_angle_list: list[float],
                 extra_info=None, point_precision: int = default_point_precision) -> None:
        super(Polygon, self).__init__(layer_name)

        self._point_precision = point_precision
        point_list = self.preprocessing_point_list(point_list, self._point_precision)
        self._point_list = point_list
        self._mid_point_list = get_rect_mid_point(self._point_list)
        self._outline_dict = {}
        extra_info = {} if extra_info is None else extra_info
        self._extra_info = extra_info

        try:
            for index in range(-1, len(point_list) - 1):
                line = Line(point_list[index][0], point_list[index][1],
                            point_list[index + 1][0], point_list[index + 1][1], layer_name, existing_angle_list)
                if abs(line.get_length()) < 0.01:
                    continue
                self._outline_dict.setdefault(line.get_angle().get_angle(), {})
                self._outline_dict[line.get_angle().get_angle()].\
                    setdefault(round(line.get_key_coord(), default_coord_key_precision), []).append(line)
        except LineLengthException as e:
            print(self._point_list)
            raise PolygonTinyAreaException(self._point_list)

        angle_list: list[float] = list(self._outline_dict.keys())
        if len(angle_list) == 0 or len(angle_list) == 1:
            raise PolygonTinyAreaException(self._point_list)

    def __setitem__(self, key, value):
        self._extra_info[key] = value

    def preprocessing_point_list(self, point_list: list[list[float]], point_precision: int) -> list[list[float]]:
        rounded_point_list: list[list[float]] = []
        last_point = None
        for coord in point_list:
            next_point = [round(x, point_precision) for x in coord]
            if calculate_distance(last_point, next_point) > 1:
                rounded_point_list.append(next_point)
                last_point = next_point
        point_list = rounded_point_list

        if len(point_list) > 4:
            new_point_list = simplify_polygon_vertices(point_list)
            if new_point_list:
                point_list = new_point_list

        points_index_to_remove: list[int] = []
        for i in range(0, len(point_list) - 1):
            if abs((point_list[i][0] - point_list[i + 1][0]) ** 2 +
                   (point_list[i][1] - point_list[i + 1][1]) ** 2) < default_minimum_point_distance:
                points_index_to_remove.append(i)
        if abs((point_list[0][0] - point_list[-1][0]) ** 2 +
               (point_list[0][1] - point_list[-1][1]) ** 2) < default_minimum_point_distance:
            points_index_to_remove.append(-1)
        for index in sorted(points_index_to_remove, reverse=True):
            del point_list[index]

        if len(point_list) < 3:
            raise PolygonTinyAreaException(self._point_list)

        points_index_to_remove: list[int] = []
        for i in range(-1, len(point_list) - 2):
            if abs(get_angle(point_list[i][0], point_list[i][1], point_list[i + 1][0], point_list[i + 1][1]) -
                   get_angle(point_list[i + 1][0], point_list[i + 1][1], point_list[i + 2][0],
                             point_list[i + 2][1])) < default_angle_threshold:
                points_index_to_remove.append(i + 1)
        if abs(get_angle(point_list[-2][0], point_list[-2][1], point_list[-1][0], point_list[-1][1]) -
               get_angle(point_list[-1][0], point_list[-1][1], point_list[0][0],
                         point_list[0][1])) < default_angle_threshold:
            points_index_to_remove.append(len(point_list) - 1)
        for index in sorted(points_index_to_remove, reverse=True):
            del point_list[index]

        return point_list

    def get_coord_list_flatten(self) -> list[float]:
        result = []
        for point in self._point_list:
            result.append(point[0])
            result.append(point[1])
        return result

    def is_rect(self) -> bool:
        is_rect = True
        outline_dict: dict[float, dict[float, list[Line]]] = self.get_outline_dict()
        for angle in outline_dict:
            if len(outline_dict[angle]) != 2:
                is_rect = False
        return is_rect

    def get_point_list(self) -> list[list[float]]:
        return self._point_list

    def get_mid_point_list(self):
        return self._mid_point_list

    def get_mid_point_list_flatten(self) -> list[float]:
        return [p for point in self._mid_point_list for p in point]

    def get_x_and_y_list(self) -> tuple[list[float], list[float]]:
        x_list: list[float] = []
        y_list: list[float] = []
        for point in self._point_list:
            x_list.append(point[0])
            y_list.append(point[1])
        x_list.append(x_list[0])
        y_list.append(y_list[0])
        return x_list, y_list

    def get_outline_dict(self) -> dict[float, dict[float, list[Line]]]:
        return self._outline_dict

    def get_angle_list(self) -> list[float]:
        return list(self._outline_dict.keys())

    def get_outlines(self) -> list[Line]:
        line_list: list[Line] = []
        for angle in self._outline_dict:
            for key in self._outline_dict[angle]:
                line_list += self._outline_dict[angle][key]
        return line_list

    def get_centroid(self) -> list[float]:
        x, y = self.get_x_and_y_list()
        return [sum(x[:-1]) / (len(x) - 1), sum(y[:-1]) / (len(y) - 1)]

    def get_width_and_length(self) -> tuple[float, float, float, Optional[Angle]]:
        temp_width = float('inf')
        temp_length = -float('inf')
        length_angle = float('inf')
        angle_file = None
        if sorted(list(self._outline_dict.keys())) == [0.0, 90.0] or \
                (len(list(self._outline_dict.keys())) == 2 and
                 abs( list(self._outline_dict.keys())[0] - list(self._outline_dict.keys())[1]) - 90 < 1):
            for angle, lines in self._outline_dict.items():
                temp_value = lines[next(iter(lines))][0].get_length()
                temp_width = temp_value if temp_width > temp_value else temp_width
                if temp_length < temp_value:
                    temp_length = temp_value
                    length_angle = angle
                    angle_file = list(lines.values())[0][0].get_angle()
            return temp_width, temp_length, length_angle, angle_file
        else:
            return temp_width, temp_length, length_angle, angle_file

    def get_extra_info(self):
        return self._extra_info

    def get_point_precision(self) -> int:
        return self._point_precision



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/20
# Author: ChunTsang <zjun5566@163.com>
"""

from src.dxfgeometry.json_geometry.json_cad_element import JsonCADElement
from src.dxfgeometry.geometry.arc import Arc


class JsonArc(JsonCADElement):
    x: float
    y: float
    start_angle: float
    end_angle: float
    radius: float

    def __init__(self, arc: Arc):
        super(JsonArc, self).__init__(arc.get_layer_name(), "ARC")
        self.x = arc.get_x()
        self.y = arc.get_y()
        self.start_angle = arc.get_start_angle()
        self.end_angle = arc.get_end_angle()
        self.radius = arc.get_radius()


def json_arc_to_arc(json_arc):
    return Arc(json_arc["x"], json_arc["y"], json_arc["start_angle"], json_arc["end_angle"], json_arc["radius"],
               json_arc["layer_name"])



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/19
# Author: ChunTsang <zjun5566@163.com>
"""

import math
from src.dxfgeometry.geometry.point import Point
from typing import Optional
from shapely.geometry import LineString as ShapelyLineString
from src.dxfgeometry.config import default_angle_precision, default_min_angle_difference, default_strong_orthogonality, \
    default_expanded_length, default_point_min_diff


def check_approximate_angle(angle: float, existing_angle_list: list[float], min_angle_difference: float,
                            strong_orthogonality: bool) -> float:
    angle_exists = False
    first_level = False
    for existing_angle in existing_angle_list:
        if abs(existing_angle - angle) < min_angle_difference or \
                abs(existing_angle - (angle - 180)) < min_angle_difference or \
                abs(existing_angle - 180 - angle) < min_angle_difference:
            if existing_angle == angle:
                angle = existing_angle
                angle_exists = True
                break
            else:
                angle = existing_angle
                angle_exists = True
                first_level = True

        elif (min_angle_difference <= abs(existing_angle - angle) < min_angle_difference * 2 or
              min_angle_difference <= abs(existing_angle - (angle - 180)) < min_angle_difference * 2 or
              min_angle_difference <= abs(existing_angle - 180 - angle) < min_angle_difference * 2) and \
                not first_level:
            if angle > existing_angle:
                angle = existing_angle + min_angle_difference * 2
                if angle >= 180:
                    angle -= 180
            elif angle < existing_angle:
                angle = existing_angle - min_angle_difference * 2
                if angle < 0:
                    angle += 180

    if not angle_exists:
        if strong_orthogonality:
            if math.isclose(angle, 90, abs_tol=min_angle_difference * 2):
                angle = 90
            elif math.isclose(angle, 0, abs_tol=min_angle_difference * 2) or \
                    math.isclose(angle, 180, abs_tol=min_angle_difference * 2):
                angle = 0
            existing_angle_list.append(angle)
        else:
            existing_angle_list.append(angle)
    return angle


class Angle(object):
    _angle: float
    _angle_in_radius: float
    _angle_precision: int
    _min_angle_difference: float
    _strong_orthogonality: bool

    def __init__(self, angle: float, existing_angle_list: list[float], angle_precision: int = default_angle_precision,
                 min_angle_difference: float = default_min_angle_difference,
                 strong_orthogonality: bool = default_strong_orthogonality) -> None:
        if angle < 0 or angle >= 180:
            raise ValueError("Angle初始化错误：角度范围错误。得到的角度为%d", angle)

        self._angle_precision = angle_precision
        self._min_angle_difference = min_angle_difference
        self._strong_orthogonality = strong_orthogonality

        angle = round(angle, self._angle_precision)
        if angle == 180:
            angle = 0

        angle = check_approximate_angle(angle, existing_angle_list, self._min_angle_difference,
                                        self._strong_orthogonality)

        self._angle = angle
        self._angle_in_radius = self._angle * (math.pi / 180)

    def get_angle(self) -> float:
        return self._angle

    def get_angle_in_radius(self) -> float:
        return self._angle_in_radius

    def get_angle_precision(self) -> int:
        return self._angle_precision

    def get_min_angle_difference(self) -> float:
        return self._min_angle_difference

    def get_strong_orthogonality(self) -> bool:
        return self._strong_orthogonality

    def get_another_point(self, ori_point: Point, length: float) -> Point:
        return Point(ori_point.get_x() + length * math.sin(self._angle_in_radius),
                     ori_point.get_y() + length * math.cos(self._angle_in_radius),
                     ori_point.get_layer_name())

    def get_mid_point(self, point: Point, length: float) -> Optional[Point]:
        return Point(point.get_x() + length * math.sin(self._angle_in_radius) / 2,
                     point.get_y() + length * math.cos(self._angle_in_radius) / 2,
                     point.get_layer_name())

    def get_point_at_dist(self, ori_point: Point, length: float, reverse: bool) -> Point:
        if not reverse:
            return Point(ori_point.get_x() + length * math.sin(self._angle_in_radius),
                         ori_point.get_y() + length * math.cos(self._angle_in_radius),
                         ori_point.get_layer_name())
        else:
            return Point(ori_point.get_x() - length * math.sin(self._angle_in_radius),
                         ori_point.get_y() - length * math.cos(self._angle_in_radius),
                         ori_point.get_layer_name())

    def get_mirroring_angle(self, mirroring_angle: "Angle", existing_angle_list: list[float]) -> float:
        mirroring_result: float = 2 * mirroring_angle.get_angle() - self._angle

        while mirroring_result >= 180:
            mirroring_result -= 180
        while mirroring_result < 0:
            mirroring_result += 180

        angle_not_exists = True
        for existing_angle in existing_angle_list:
            if abs(existing_angle - mirroring_result) < self._min_angle_difference or \
                    abs(existing_angle - (mirroring_result - 180)) < self._min_angle_difference:
                mirroring_result = existing_angle
                angle_not_exists = False
        if angle_not_exists:
            existing_angle_list.append(mirroring_result)

        return mirroring_result

    def get_ortho_angle(self, existing_angle_list: list[float]) -> "Angle":
        ortho_angle = (self.get_angle() + 90) % 180
        return Angle(ortho_angle, existing_angle_list)

    def get_key_coord_len_coord_of_point(self, point: Point) -> tuple[float, float]:
        one_point = self.get_point_at_dist(point, default_expanded_length, True)
        another_point = self.get_point_at_dist(point, default_expanded_length, False)

        expanded_line: ShapelyLineString = ShapelyLineString([[one_point.get_x(), one_point.get_y()],
                                                              [another_point.get_x(), another_point.get_y()]])

        angle_in_radius: float = self._angle_in_radius + math.pi / 2
        perpendicular_line: ShapelyLineString = ShapelyLineString([
            [default_expanded_length * math.sin(angle_in_radius),
             default_expanded_length * math.cos(angle_in_radius)],
            [-default_expanded_length * math.sin(angle_in_radius),
             -default_expanded_length * math.cos(angle_in_radius)]
        ])

        intersection_point = perpendicular_line.intersection(expanded_line)
        x_coord, y_coord = list(intersection_point.coords)[0]

        if (abs(y_coord) < default_point_min_diff and
            x_coord < -default_point_min_diff) or \
                y_coord > default_point_min_diff:
            key_coord = - math.sqrt(x_coord ** 2 + y_coord ** 2)
        else:
            key_coord = math.sqrt(x_coord ** 2 + y_coord ** 2)

        if self._angle < 90:
            if point.get_x() > x_coord or point.get_y() > y_coord:
                length_coord = math.sqrt((point.get_x() - x_coord) ** 2 + (point.get_y() - y_coord) ** 2)
            else:
                length_coord = -math.sqrt((point.get_x() - x_coord) ** 2 + (point.get_y() - y_coord) ** 2)
        else:
            if point.get_x() > x_coord or point.get_y() < y_coord:
                length_coord = math.sqrt((point.get_x() - x_coord) ** 2 + (point.get_y() - y_coord) ** 2)
            else:
                length_coord = -math.sqrt((point.get_x() - x_coord) ** 2 + (point.get_y() - y_coord) ** 2)

        return key_coord, length_coord




# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/20
# Author: ChunTsang <zjun5566@163.com>
"""

from src.dxfgeometry.json_geometry.json_cad_element import JsonCADElement
from src.dxfgeometry.geometry.point import Point


class JsonPoint(JsonCADElement):
    x: float
    y: float
    
    def __init__(self, point: Point):
        super(JsonPoint, self).__init__(point.get_layer_name(), "Point")
        self.x = point.get_x()
        self.y = point.get_y()


def json_point_to_point(json_point: dict) -> Point:
    return Point(json_point['x'], json_point['y'], json_point['layer_name'])



# -*-coding:utf-8 -*-
"""
# Time  : 2026/1/20
# Author: ChunTsang <zjun5566@163.com>
"""

from src.dxfgeometry.json_geometry.json_cad_element import JsonCADElement
from src.dxfgeometry.geometry.circle import Circle


class JsonCircle(JsonCADElement):
    x: float
    y: float
    radius: float

    def __init__(self, circle: Circle):
        super(JsonCircle, self).__init__(circle.get_layer_name(), "Circle")
        self.x = circle.get_x()
        self.y = circle.get_y()
        self.radius = circle.get_radius()
        self.electric_info = {}

    def __setitem__(self, key, value):
        self.electric_info[key] = value


def json_circle_to_circle(json_circle: dict) -> Circle:
    return Circle(json_circle['x'], json_circle['y'], json_circle['radius'], json_circle['layer_name'])
